<?php

if (!defined('ABSPATH')) exit;

class sap_playlists_log extends WP_List_Table {
    function __construct($args = array()) {
        parent::__construct(array(
            'singular'=> 'playlist',
            'plural' => 'playlists',
            'ajax' => false
        ));
    }

    function get_table_classes() {
        return array('widefat', 'fixed', $this->_args['plural'], 'sap-play-log');
    }

    function extra_tablenav($which) {
        if ($which == 'top') {
            echo '<div class="alignleft actions">';
            $this->months_dropdown();

            submit_button(__("Filter", "smart-audio-playlists"), 'button', false, false, array('id' => 'log-query-submit'));
            echo '</div>';
        }
    }

    function months_dropdown() {
        global $wpdb, $wp_locale;

        $sql = "SELECT DISTINCT YEAR(played) AS year, MONTH(played) AS month FROM ".$wpdb->prefix."sap_play_log ORDER BY played DESC";
        $months = $wpdb->get_results($sql);

        $list = array('0' => __("All dates", "smart-audio-playlists"));

        foreach ($months as $row) {
            $month = zeroise($row->month, 2);
            $year = $row->year;

            $list[$year.'-'.$month] = sprintf(__("%s %s", "smart-audio-playlists"), $wp_locale->get_month($month), $year);
        }

        $filter_date = isset($_GET['filter-date']) ? $_GET['filter-date'] : 0;

        sap_render_select($list, array('selected' => $filter_date, 'class' => '', 'name' => 'filter-date'));
    }
    
    function rows_per_page() {
        $user = get_current_user_id();
        $per_page = get_user_meta($user, 'sap_rows_per_page_log', true);

        if (empty($per_page) || $per_page < 1) {
            $per_page = 25;
        }

        return $per_page;
    }

    function get_columns() {
	return array(
            'cb' => '<input type="checkbox" />',
            'id' => __("ID", "smart-audio-playlists"),
            'thumb' => '',
            'playlist' => __("Playlist", "smart-audio-playlists"),
            'playtime' => __("Playtime", "smart-audio-playlists"),
            'tracks' => '<span title="Tracks" class="dashicons dashicons-media-audio"></span>',
            'time' => '<span title="Time" class="dashicons dashicons-clock"></span>',
            'source' => __("Source", "smart-audio-playlists"),
            'ip' => __("IP and User", "smart-audio-playlists"),
            'played' => __("Date", "smart-audio-playlists")
	);
    }

    function get_sortable_columns() {
	return array(
            'id' => array('l.id', false),
            'playlist' => array('p.post_title', false)
	);
    }

    function get_bulk_actions() {
        $bulk = array(
            'delete' => __("Delete", "smart-audio-playlists")
        );

        return $bulk;
    }

    function single_row($item) {
        static $row_class = '';
        $row_class = $row_class == '' ? ' alternate' : '';

        echo '<tr class="'.$row_class.'">';
        $this->single_row_columns($item);
        echo '</tr>';
    }

    function column_cb($item) {
        return sprintf('<input type="checkbox" name="%1$s[]" value="%2$s" />', $this->_args['singular'], $item->id);
    }

    function column_playtime($item) {
        $min = floor($item->timer / 60);
        $sec = $item->timer - $min * 60;

        $min = $min < 10 ? '0'.$min : $min;
        $sec = $sec < 10 ? '0'.$sec : $sec;

        $per = floor(($item->timer / sap_playlist($item->post_id)->total_time) * 100);
        
        $render = '<strong>'.$item->plays.'</strong> '._n("song", "songs", $item->plays, "smart-audio-playlists").' '.__("played", "smart-audio-playlists").'<br/>';
        $render.= '<strong>'.$min.':'.$sec.'</strong> '.__("playtime", "smart-audio-playlists").'<br/>';
        $render.= '<strong>'.$per.'%</strong> '.__("played", "smart-audio-playlists");

        return $render;
    }

    function column_thumb($item){
        $thumbnail = get_the_post_thumbnail($item->post_id, array(60, 60));

        if ($thumbnail == '') {
            return __("None", "smart-audio-playlists");
        } else {
            return $thumbnail;
        }
    }

    function column_playlist($item) {
        $actions = array(
            'delete' => sprintf('<a class="sap-delete-from-log" href="edit.php?post_type=sapplaylist&page=smart-audio-playlists-log&single-action=delete&log=%s&_wpnonce=%s">%s</a>', $item->id, wp_create_nonce('smart-audio-playlists-log'), __("Delete Log Entry", "smart-audio-playlists")),
            'list' => sprintf('<a href="edit.php?post_type=sapplaylist&page=smart-audio-playlists-log&filter-list=%s">%s</a>', $item->post_id, __("Log for this playlist only", "smart-audio-playlists")),
            'tracks' => sprintf('<a class="sap-open-tracks-log" href="#" data-log="%s" data-playlist="%s">%s</a>', $item->id, $item->post_id, __("Tracks log", "smart-audio-playlists")),
            'edit' => sprintf('<a href="post.php?post=%s&action=edit">%s</a>', $item->post_id, __("Edit", "smart-audio-playlists")),
            'view' => sprintf('<a href="%s">%s</a>', get_permalink($item->post_id), __("View", "smart-audio-playlists"))
        );

        return '<strong>'.$item->post_title.'</strong>'.$this->row_actions($actions);
    }

    function column_played($item) {
        $timestamp = strtotime($item->played);

        return date('Y-m-d', $timestamp).'<br/>@ '.date('H:i:s', $timestamp);
    }

    function column_tracks($item) {
        return count(sap_playlist($item->post_id)->tracks);
    }

    function column_time($item) {
        return sap_playlist($item->post_id)->running_time();
    }

    function column_source($item) {
        $render = '';

        switch ($item->source) {
            case 'widget':
                $render = __("Widget", "smart-audio-playlists");
                break;
            case 'shortcode':
                $render = __("Shortcode", "smart-audio-playlists");
                break;
            default:
            case 'page':
                $render = __("Page", "smart-audio-playlists");
                break;
        }
        
        return $render;
    }

    function column_ip($item) {
        $render = $item->ip.'<br/>';

        if ($item->user_id == 0) {
            $render.= __("Visitor", "smart-audio-playlists");
        } else {
            $userdata = get_userdata($item->user_id);
            $render.= '<a href="'.get_edit_user_link($item->user_id).'">'.esc_attr($userdata->user_nicename).'</a>';
        }

        return $render;
    }

    function column_default($item, $column_name) {
        return $item->$column_name;
    }

    function prepare_items() {
        global $wpdb;

        $this->_column_headers = array($this->get_columns(), array(), $this->get_sortable_columns());

        $per_page = $this->rows_per_page();

        $table = $wpdb->prefix.'sap_play_log';

        $query_where = array("l.type = 'playlist'", "l.plays > 0");
        $query_subscribers = "SELECT SQL_CALC_FOUND_ROWS l.*, p.post_title FROM ".$table." l";
        $query_subscribers.= " INNER JOIN ".$wpdb->posts." p ON p.ID = l.post_id";

        if (isset($_GET['s']) && $_GET['s'] != '') {
            $query_where[] = "(p.post_title LIKE '%".$_GET['s']."%')";
        }

        if (isset($_GET['filter-date']) && $_GET['filter-date'] != '' && $_GET['filter-date'] != '0') {
            $date = explode('-', $_GET['filter-date']);

            if (count($date) == 2) {
                $query_where[] = "YEAR(played) = ".intval($date[0]);
                $query_where[] = "MONTH(played) = ".intval($date[1]);
            }
        }

        if (isset($_GET['filter-list']) && $_GET['filter-list'] != '') {
            $query_where[] = "l.post_id = ".intval($_GET['filter-list']);
        }

        if (!empty($query_where)) {
            $query_subscribers.= ' WHERE '.join(' AND ', $query_where);
        }

        $orderby = !empty($_GET['orderby']) ? mysql_real_escape_string($_GET['orderby']) : 'l.id';
        $order = !empty($_GET['order']) ? mysql_real_escape_string($_GET['order']) : 'DESC';

        if (!empty($orderby) && !empty($order)) {
            $query_subscribers.= " ORDER BY $orderby $order";
        } else {
            $query_subscribers.= " ORDER BY `id` DESC";
        }

        $paged = !empty($_GET['paged']) ? mysql_real_escape_string($_GET['paged']) : '';
        if (empty($paged) || !is_numeric($paged) || $paged <= 0 ){
            $paged = 1;
        }

        $offset = intval(($paged - 1) * $per_page);
        $query_subscribers.= " LIMIT $offset, $per_page";

        $this->items = $wpdb->get_results($query_subscribers);

        $total_rows = $wpdb->get_var('SELECT FOUND_ROWS()');

        $this->set_pagination_args(array(
            'total_items' => $total_rows,
            'total_pages' => ceil($total_rows / $per_page),
            'per_page' => $per_page,
        ));
    }
}