<?php

if (!defined('ABSPATH')) exit;

class sap_objects {
    public static function register_genre() {
        $reg = array(
            'labels' => sap_objects::taxonomy_labels(
                    smart_sap_core()->get('tax_genre_singular'), 
                    smart_sap_core()->get('tax_genre_plural')),
            'hierarchical' => false, 
            'rewrite' => array(
                'hierarchical' => false,
                'with_front' => true,
                'slug' => smart_sap_core()->get('cpt_playlist_slug_archive').'/'.smart_sap_core()->get('tax_genre_slug_single')),
            'query_var' => true,
            'public' => true,
            'show_ui' => true,
            'show_tagcloud' => true,
            'show_admin_column' => true, 
            'capabilities' => array(
                'manage_terms' => 'manage_categories',
                'edit_terms' => 'manage_categories',
                'delete_terms' => 'manage_categories',
                'assign_terms' => 'manage_categories'),
            'show_in_nav_menus' => true
        );

        $data = apply_filters('sap_register_taxonomy_genre', $reg);

        register_taxonomy('sapgenre', array('sapplaylist'), $data);
    }

    public static function register_artist() {
        $reg = array(
            'labels' => sap_objects::taxonomy_labels(
                    smart_sap_core()->get('tax_artist_singular'), 
                    smart_sap_core()->get('tax_artist_plural')),
            'hierarchical' => false, 
            'rewrite' => array(
                'hierarchical' => false,
                'with_front' => true,
                'slug' => smart_sap_core()->get('cpt_playlist_slug_archive').'/'.smart_sap_core()->get('tax_artist_slug_single')),
            'query_var' => true,
            'public' => true,
            'show_ui' => true,
            'show_tagcloud' => true,
            'show_admin_column' => true, 
            'capabilities' => array(
                'manage_terms' => 'manage_categories',
                'edit_terms' => 'manage_categories',
                'delete_terms' => 'manage_categories',
                'assign_terms' => 'manage_categories'),
            'show_in_nav_menus' => true
        );

        $data = apply_filters('sap_register_taxonomy_artist', $reg);

        register_taxonomy('sapartist', array('sapplaylist'), $data);
    }

    public static function register_track() {
        $reg = array(
            'labels' => sap_objects::posttype_labels(
                    smart_sap_core()->get('cpt_track_singular'), 
                    smart_sap_core()->get('cpt_track_plural')),
            'publicly_queryable' => false,
            'exclude_from_search' => true,
            'capability_type' => 'post',
            'hierarchical' => false,
            'public' => false,
            'rewrite' => false,
            'show_in_menu' => false,
            'show_in_admin_bar' => false,
            'has_archive' => false,
            'query_var' => false,
            'supports' => array(),
            'taxonomies' => array(),
            'show_ui' => false,
            'can_export' => true,
            'show_in_nav_menus' => false
        );

        $data = apply_filters('sap_register_post_type_track', $reg);

        register_post_type('saptrack', $data);
    }

    public static function register_playlist() {
        $menu_icon = SAP_WP_VERSION == 38 ? 'dashicons-format-audio' : 'dashicons-playlist-audio';
        $taxonomies = smart_sap_core()->get('cpt_playlist_tax_tag') ? array('post_tag') : array();

        $reg = array(
            'labels' => sap_objects::posttype_labels(
                    smart_sap_core()->get('cpt_playlist_singular'), 
                    smart_sap_core()->get('cpt_playlist_plural')),
            'publicly_queryable' => true,
            'exclude_from_search' => false,
            'capability_type' => 'post',
            'hierarchical' => false,
            'public' => true,
            'rewrite' => array(
                'slug' => smart_sap_core()->get('cpt_playlist_slug_single'),
                'with_front' => false,
                'feeds' => true,
                'pages' => true),
            'show_in_menu' => true,
            'show_in_admin_bar' => true,
            'has_archive' => smart_sap_core()->get('cpt_playlist_slug_archive'),
            'query_var' => true,
            'supports' => array(
                'title',
                'editor',
                'author',
                'thumbnail',
                'excerpt',
                'comments',
                'revisions'),
            'taxonomies' => $taxonomies,
            'show_ui' => true,
            'can_export' => true,
            'show_in_nav_menus' => true,
            'menu_icon' => $menu_icon,
            '_edit_link' => 'post.php?post=%d'
        );

        $data = apply_filters('sap_register_post_type_playlist', $reg);

        register_post_type('sapplaylist', $data);
    }

    public static function posttype_labels($singular, $plural) {
        $labels = array(
            'name' => $plural,
            'singular_name' => $singular,
            'menu_name' => $plural
        );

        $labels['add_new'] = __("Add New", "smart-audio-playlists");
        $labels['edit'] = __("Edit", "smart-audio-playlists");
        $labels['add_new_item'] = __("Add New", "smart-audio-playlists").' '.$singular;
        $labels['edit_item'] = __("Edit", "smart-audio-playlists").' '.$singular;
        $labels['new_item'] = __("New", "smart-audio-playlists").' '.$singular;
        $labels['view_item'] = __("View", "smart-audio-playlists").' '.$singular;
        $labels['search_items'] = __("Search", "smart-audio-playlists").' '.$plural;
        $labels['not_found'] = __("No", "smart-audio-playlists").' '.$plural.' '.__("Found", "smart-audio-playlists");
        $labels['not_found_in_trash'] = __("No", "smart-audio-playlists").' '.$plural.' '.__("Found In Trash", "smart-audio-playlists");
        $labels['parent_item_colon'] = __("Parent", "smart-audio-playlists").' '.$plural.':';
        $labels['all_items'] = __("All", "smart-audio-playlists").' '.$plural;
        $labels['menu_name'] = $plural;

        return $labels;
    }

    public static function taxonomy_labels($singular, $plural) {
        $labels = array(
            'name' => $plural,
            'singular_name' => $singular,
            'menu_name' => $plural
        );

        $labels['parent_item'] = __("Parent", "smart-audio-playlists").' '.$singular;
        $labels['search_items'] = __("Search", "smart-audio-playlists").' '.$plural;
        $labels['popular_items'] = __("Popular", "smart-audio-playlists").' '.$plural;
        $labels['all_items'] = __("All", "smart-audio-playlists").' '.$plural;
        $labels['edit_item'] = __("Edit", "smart-audio-playlists").' '.$singular;
        $labels['view_item'] = __("View", "smart-audio-playlists").' '.$singular;
        $labels['update_item'] = __("Update", "smart-audio-playlists").' '.$singular;
        $labels['add_new_item'] = __("Add New", "smart-audio-playlists").' '.$singular;
        $labels['add_or_remove_items'] = __("Add or remove", "smart-audio-playlists").' '.$plural;
        $labels['choose_from_most_used'] = __("Choose from the most used", "smart-audio-playlists").' '.$plural;
        $labels['parent_item_colon'] = __("Parent", "smart-audio-playlists").' '.$plural.':';
        $labels['new_item_name'] = __("New", "smart-audio-playlists").' '.$plural.' '.__("Name", "smart-audio-playlists");
        $labels['not_found'] = __("No", "smart-audio-playlists").' '.$plural.' '.__("found", "smart-audio-playlists");
        $labels['separate_items_with_commas'] = __("Separate", "smart-audio-playlists").' '.$plural.' '.__("with commas", "smart-audio-playlists");

        return $labels;
    }

    public static function posttype_archive_slug($post_type) {
        $slug_archive = $post_type->has_archive;

        if ($slug_archive === true) {
            $slug_archive = $post_type->name;
        }

        return $slug_archive;
    }

    public static function rewrite_posttype_date_archives($cpt, $wp_rewrite) {
        $rules = array();
        $post_type = get_post_type_object($cpt);
        $slug_archive = sap_objects::posttype_archive_slug($post_type);

        $dates = array(
            array('rule' => "([0-9]{4})/([0-9]{1,2})/([0-9]{1,2})", "vars" => array('year', 'monthnum', 'day')),
            array('rule' => "([0-9]{4})/([0-9]{1,2})", "vars" => array('year', 'monthnum')),
            array('rule' => "([0-9]{4})", "vars" => array('year'))
        );

        foreach ($dates as $data) {
            $query = 'index.php?post_type='.$cpt;
            $rule = $slug_archive."/".$data['rule'];

            $i = 1;
            foreach ($data['vars'] as $var) {
                $query.= '&'.$var.'='.$wp_rewrite->preg_index($i);
                $i++;
            }

            $rules[$rule."/?$"] =                                   $query;
            $rules[$rule."/feed/(feed|rdf|rss|rss2|atom)/?$"] =     $query.'&feed='.$wp_rewrite->preg_index($i);
            $rules[$rule."/(feed|rdf|rss|rss2|atom)/?$"] =          $query.'&feed='.$wp_rewrite->preg_index($i);
            $rules[$rule."/page/([0-9]{1,})/?$"] =                  $query.'&paged='.$wp_rewrite->preg_index($i);
        }

        return $rules;
    }

    public static function rewrite_posttype_taxonomies($cpt, $taxonomies, $wp_rewrite) {
        $rules = array();

        $post_type = get_post_type_object($cpt);
        $slug_archive = sap_objects::posttype_archive_slug($post_type);

        if (is_string($slug_archive)) {
            $slug_archive.= '/';

            foreach ($taxonomies as $taxonomy) {
                if (taxonomy_exists($taxonomy)) {
                    $tax = get_taxonomy($taxonomy);

                    $slug = isset($tax->rewrite['slug']) ? $tax->rewrite['slug'] : '';
                    $query_var = $tax->query_var === true ? $tax->name : ($tax->query_var !== false ? $tax->query_var : $tax->name);

                    if (!empty($slug)) {
                        $slug_sapindex = smart_sap_core()->tax_archive_slug($taxonomy);

                        if ($slug_sapindex != '') {
                            $slug_sapindex.= '/';

                            $rules[$slug_archive.$slug_sapindex."?$"] =                          "index.php?post_type=".$cpt."&sapindex=".$taxonomy;
                            $rules[$slug_archive.$slug_sapindex."page/([0-9]{1,})/?$"] =         "index.php?post_type=".$cpt."&sapindex=".$taxonomy."&paged=".$wp_rewrite->preg_index(1);
                        }

                        $slug = end(explode('/', $slug)).'/';

                        $rules[$slug_archive.$slug."([^/]+)/?$"] =                               "index.php?post_type=".$cpt."&".$query_var."=".$wp_rewrite->preg_index(1);
                        $rules[$slug_archive.$slug."([^/]+)/feed/(feed|rdf|rss|rss2|atom)/?$"] = "index.php?post_type=".$cpt."&feed=".$wp_rewrite->preg_index(2)."&".$query_var."=".$wp_rewrite->preg_index(1);
                        $rules[$slug_archive.$slug."([^/]+)/(feed|rdf|rss|rss2|atom)/?$"] =      "index.php?post_type=".$cpt."&feed=".$wp_rewrite->preg_index(2)."&".$query_var."=".$wp_rewrite->preg_index(1);
                        $rules[$slug_archive.$slug."([^/]+)/page/([0-9]{1,})/?$"] =              "index.php?post_type=".$cpt."&paged=".$wp_rewrite->preg_index(2)."&".$query_var."=".$wp_rewrite->preg_index(1);
                    }
                }
            }

            $rules[$slug_archive."/page/([0-9]{1,})/?$"] = "index.php?post_type=".$cpt."&paged=".$wp_rewrite->preg_index(1);
        }

        return $rules;
    }
}

?>